#include "mpi.h"
#include <stdlib.h>

#define COLS 1000
#define ROWS 1000

int inset( double ix, double iy )
{
   int iterations = 0;
   double x = ix, y = iy, x2 = x*x, y2 = y*y;
   while ( ( x2 + y2 < 4 ) && ( iterations < 1000 ) )
   {
     y  = 2 * x * y + iy;
     x  = x2 - y2 + ix;
     x2 = x * x;
     y2 = y * y;
     iterations++;
   }
   return iterations; 
}

void computerow( int index, int * data )
{
  double y = -1.5 + (double)index / ROWS;
  for( int c=0; c<COLS; c++ )
  {
    double x = -1.5 + (double)c / COLS;
    data[c] = inset( x, y );
  }
}

int numproc, myid, rows, cols;


void masterprocess()
{
  int ** data;
  MPI_Status status;
  data = malloc( sizeof(int*) * ROWS ); // Allocate memory to store data
  data[0] = malloc( sizeof(int) * COLS ); 
  computerow( 0, data[0] );             // Compute row zero
  for( int currentrow = 1; currentrow < rows; currentrow++ )
  { 
    data[currentrow] = malloc( sizeof(int) * cols ); 
    int process = currentrow % ( numproc - 1 ) + 1;
    MPI_Recv( data[currentrow], COLS, MPI_INT, process, currentrow, 
MPI_COMM_WORLD, &status );
  }
}

void workerprocess()
{
  int * data, row;
  data = malloc( sizeof(int) * COLS );
  row = myid;
  while ( row <= ROWS )
  {
    computerow( row, data );
    MPI_Send( data, COLS, MPI_INT, 0, row, MPI_COMM_WORLD );
    row += numproc-1;
  }
  free( data );
}
  

int main( int argc, char *argv[] ) 
{
  MPI_Init( &argc, &argv );
  MPI_Comm_size( MPI_COMM_WORLD, &numproc );
  MPI_Comm_rank( MPI_COMM_WORLD, &myid );
  if ( myid == 0 ) { masterprocess(); } else { workerprocess(); }
  MPI_Finalize();
  return 0;
} 
