#include <stdlib.h>
#include <strings.h>
#include <pthread.h>

#define ACCOUNTS 256
#define BRANCHES 128
int account[BRANCHES][ACCOUNTS];
pthread_mutex_t mutex[BRANCHES];

void move( int branch, int from, int to, int value )
{
  pthread_mutex_lock( &mutex[branch] );
  account[branch][from] -= value;
  account[branch][to] += value;
  pthread_mutex_unlock( &mutex[branch] );
}

void *experiment( void *param )
{
  unsigned int seed = 0;
  int count = 10000000 / (int)param;
  for( int i=0; i<count; i++ )
  {
     int  row = rand_r(&seed) & (BRANCHES-1);
     int from = rand_r(&seed) & (ACCOUNTS-1);
     int   to = rand_r(&seed) & (ACCOUNTS-1);
     move( row, from, to, 1 );
  }
}

int main( int argc, char* argv[] )
{
  pthread_t threads[64];
  memset( account, 0, sizeof(account) );
  int nthreads = 8;
  if ( argc > 1 ) { nthreads = atoi( argv[1] ); }
  for( int i=0; i<BRANCHES; i++ )
    pthread_mutex_init( &mutex[i], 0 );
  for( int i=0; i<nthreads; i++ )
  {
    pthread_create( &threads[i], 0, experiment, (void*)nthreads );
  }
  for( int i=0; i<nthreads; i++ )
  {
    pthread_join( threads[i], 0 );
  }
  for( int i=0; i<BRANCHES; i++ )
    pthread_mutex_destroy( &mutex[i] );
  return 0;
}
